import java.sql.Timestamp;
import java.util.Hashtable;
import java.util.Map;

public class TimedCache {

	private Map<Object, TimestampedValue> cachedValues = new Hashtable<Object, TimestampedValue>();

	private class TimestampedValue {
		public Object value;
		public Timestamp loadTime;
	}

	private ObjectLoader loader;
	private Clock clock;
	private ReloadPolicy policy;

	public TimedCache(ObjectLoader loader, Clock clock, ReloadPolicy policy) {
		this.loader = loader;
		this.clock = clock;
		this.policy = policy;
	}

	public Object lookUp(Object key) {
		TimestampedValue timestampedValue = (TimestampedValue) cachedValues
				.get(key);
		Timestamp currentTime = clock.getCurrentTime();
		if ((timestampedValue == null)
				|| ((policy
						.shouldReload(timestampedValue.loadTime, currentTime)))) {
			Object value = loader.load(key);
			timestampedValue = new TimestampedValue();
			timestampedValue.loadTime = currentTime;
			timestampedValue.value = value;
			cachedValues.put(key, timestampedValue);
		}
		return timestampedValue.value;
	}
}
